/**
  @file
  @brief testing postdata with mpe_x_test

  <h4> SAS Macros </h4>
  @li mp_testservice.sas
  @li mp_assert.sas


**/


/**
  * First part - stage some data (for diffing)
  */
data work.sascontroltable;
  action='LOAD';
  message="getdiffs prep";
  libds="&dclib..MPE_X_TEST";
  output;
  stop;
run;

proc sql noprint;
select max(primary_key_field) into: maxpk
  from &dclib..mpe_x_test;

data work.jsdata;
  set &dclib..mpe_x_test(rename=(
    some_date=dt2 SOME_DATETIME=dttm2 SOME_TIME=tm2)
  );
  /* for now, the adapter sends these as strings */
  some_date=put(dt2,date9.);
  SOME_DATETIME=put(dttm2,datetime19.);
  some_time=put(tm2,time.);
  drop dt2 dttm2 tm2;
  _____DELETE__THIS__RECORD_____='No';
  if _n_=1 then do;
    primary_key_field=sum(&maxpk,1);
    some_char='   leadingblanks';
    some_num=._;
    output;
  end;
  else stop;
run;

%mp_testservice(&appLoc/services/editors/stagedata,
  viyacontext=&defaultcontext,
  inputdatasets=work.jsdata work.sascontroltable,
  outlib=web1
)

%let status=0;
data work.sasparams;
  set web1.sasparams;
  putlog (_all_)(=);
  if status='SUCCESS' then call symputx('status',1);
  call symputx('dsid',dsid);
run;

%mp_assert(
  iftrue=(&status=1 and &syscc=0),
  desc=Checking successful submission
)

/**
  * Now run postdata with SHOW_DIFFS
  */
data work.sascontroltable;
  ACTION='SHOW_DIFFS';
  TABLE="&dsid";
  DIFFTIME="%sysfunc(datetime(),B8601DT19.3)";
  output;
  stop;
run;
%mp_testservice(&appLoc/services/auditors/postdata,
  viyacontext=&defaultcontext,
  inputdatasets=work.sascontroltable,
  outlib=web2,
  outref=wbout
)

%let leadcheck=0;
%let speshcheck=0;
data _null_;
  infile wbout;
  input;
  putlog _infile_;
  /* the JSON libname option removes leading blanks!!!! */
  if index(_infile_,'   leadingblanks') then call symputx('leadcheck',1);
  /* there is no clean way to send a special missing - so is sent as string */
  if index(_infile_,',"SOME_NUM":"_"') then call symputx('speshcheck',1);
run;


%mp_assert(
  iftrue=(&leadcheck=1),
  desc=Checking leading blanks were applied
)
%mp_assert(
  iftrue=(&leadcheck=1),
  desc=Checking special characters were applied
)
